@@ -5,12 +5,10 @@ import {
 } from '@jupyterlab/filebrowser';
 import { ITranslator } from '@jupyterlab/translation';
 import {
-  showErrorMessage,
+  showErrorMessage, Spinner
 } from '@jupyterlab/apputils';
 import { PathExt } from '@jupyterlab/coreutils';
 
-import { ContentsManager } from '@jupyterlab/services';
-
 import { DisposableDelegate } from '@lumino/disposable';
 
 import { ToolbarButton } from '@jupyterlab/apputils';
@@ -68,37 +66,37 @@ async function parseURL(queryString, app, browser, trans){
       const req = await fetch(url);
       blob = await req.blob();
       type = req.headers.get('Content-Type') ?? '';
+        // upload the content of the file to the server
+      try {
+        const name = PathExt.basename(url).split("?")[0];
+        const file = new File([blob], name, { type });
+        const model = await browser.model.upload(file);
+        lastFile = model.path
+        console.log("File uploaded: " + model.path);
+      } catch (error) {
+        showErrorMessage(
+          trans._p('showErrorMessage', 'Upload Error'),
+          error
+        );
+      }
     } catch (reason) {
       if (reason.response && reason.response.status !== 200) {
         reason.message = trans.__('Could not open URL: %1', url);
       }
       return showErrorMessage(trans.__('Cannot fetch'), reason);
     }
-    // upload the content of the file to the server
-    try {
-      const name = PathExt.basename(url).split("?")[0];
-      const file = new File([blob], name, { type });
-      const model = await browser.model.upload(file);
-      lastFile = model.path
-      console.log("File uploaded: " + model.path);
-    } catch (error) {
-      return showErrorMessage(
-        trans._p('showErrorMessage', 'Upload Error'),
-        error
-      );
-    }
   }
 
   let file2Open = urlParams.get("open") 
   if(file2Open === "1" ) file2Open = lastFile;
   if(file2Open){
-    app.commands.execute('docmanager:open', {
+    await app.commands.execute('docmanager:open', {
       path: file2Open
     });
   }
 }
 class ImjoyExtension {
-  constructor(jupyterBaseUrl) {
+  constructor(jupyterBaseUrl, labApp, fileBrowser) {
     this.baseUrl = jupyterBaseUrl;
     const isIframe = window.self !== window.top;
     this.notebookHandlerPromise = new Promise((resolve, reject) => {
@@ -141,23 +139,19 @@ class ImjoyExtension {
       imjoy = app.imjoy;
       const kernelInfo = {}
       if(isIframe){
-        const contents = new ContentsManager();
         const api = await setupRPC({ name: "ImJoy" });
         api.export({
           setup(){
           },
-          async saveContent(name, content){
-            await contents.save(name, content);
+          async loadFile(name, content, type){
+            const file = new File([content], name, { type });
+            const model = await fileBrowser.model.upload(file);
+            return model.path;
           },
-          async openUrl(url, with_token){
-            if(with_token){
-              if(url.indexOf('?') === -1){
-                url = url + '?token=' + contents.serverSettings.token;
-              }else{
-                url = url + '&token=' + contents.serverSettings.token;
-              }
-            }
-            window.location.href = url;
+          async openFile(path){
+            await labApp.commands.execute('docmanager:open', {
+              path: path
+            });
           }
         })
       }
@@ -222,10 +216,28 @@ class ImjoyExtension {
 
       this.resolveNotebookHandler(async (sessionContext, panelNode, buttonNode)=>{
         const { kernel } = sessionContext.session;
+        buttonNode.style.minWidth = "25px";
+        const _spinner = new Spinner();
+        _spinner.node.firstChild.style.width = "20px";
+        _spinner.node.firstChild.style.height = "20px";
         sessionContext.kernelChanged.connect(() => {
-          patchPyodideKernel(kernel);
+          buttonNode.appendChild(_spinner.node);
+          patchPyodideKernel(kernel).finally(()=>{
+            buttonNode.removeChild(_spinner.node);
+          }).catch(()=>{
+            console.error("Failed to apply pyodide patch");
+          })
         }, sessionContext);
-        await patchPyodideKernel(kernel);
+        buttonNode.appendChild(_spinner.node);
+        try{
+          await patchPyodideKernel(kernel);
+        }
+        catch(e){
+          throw e;
+        }
+        finally{
+          buttonNode.removeChild(_spinner.node);
+        }
         kernelInfo[kernel._id] = { kernel };
         buttonNode.firstChild.innerHTML = `<img src="https://imjoy.io/static/img/imjoy-logo-black.svg" style="height: 17px;">`;
         buttonNode.firstChild.onclick = () => {
@@ -270,7 +282,7 @@ export default {
     app.serviceManager.settings.serverSettings.baseUrl;
     app.docRegistry.addWidgetExtension(
       'Notebook',
-      new ImjoyExtension(jupyterBaseUrl),
+      new ImjoyExtension(jupyterBaseUrl, app, browser),
     );
     console.log(
       'JupyterLab extension imjoy-jupyterlab-extension is activated!'
